#pragma once
#include "ForwardDef.h"
#include "WMAllocator.h"

#include "CurveTypes.h"
namespace WorldMachine {

// A few basic curve types for interpolating between various values
class Curve1DConstant : public Curve1DInterface {
	float constval;
public:
	Curve1DConstant(float val);

	// 1D Constant is meant to be used as a default value allocated on the stack only; Release() does nothing here!
	virtual bool Release();
	virtual Curve1DInterface *Clone();
	virtual double evaluatePoint(double x);
	virtual bool isNormalized();
	virtual bool isExtendedDomain();
	virtual Vector2 getRange();
	virtual Vector2 getDomain();
};

// A 1D curve intended for interpolating sets of datapoints.
// You can use linear or automatic cubic curve fitting (optionally restricting to monotonic interpolation)
class Curve1DSpline : public Curve1DInterface {
public:
	struct PointType {
		PointType() { x = 0.0; y = 0.0; der = 0.0; };
		PointType(double x_, double y_) { x = x_; y = y_; der = 0.0;};
		double x;
		double y;
		double der;
	};
	enum CURVESPLINETYPE { LINEAR, AUTOCUBIC, MONOTONECUBIC };
	enum BOUNDARY_DER { NATURAL, SPECIFIED };
	Curve1DSpline(CURVESPLINETYPE linetype); // 0 = polyline, 1 = auto-bezier

	void Load(std::istream &in);
	void Save(std::ostream &out);

	virtual bool Release();
	virtual Curve1DInterface *Clone();
	virtual double evaluatePoint(double x) override;
	virtual bool isNormalized() override;
	virtual bool isExtendedDomain() override;
	virtual Vector2 getRange() override;
	virtual Vector2 getDomain() override;

	int numPoints() const;
	void addPoint(double x, double y);
	void clearPoints();

	PointType getPoint(int i) const;
	void changePoint(int i, PointType newpoint);

	CURVESPLINETYPE getInterpolationType() const { return interpolation_type; };
	void setInterpolationType(CURVESPLINETYPE newtype) { interpolation_type = newtype; };

	void setBoundaryCondition(BOUNDARY_DER type, float der);

	void enforceConstraints();
private:
	std::vector<PointType, wmcore_allocator<PointType>> points;
	Vector2 boundary_der;
	CURVESPLINETYPE interpolation_type;
};

}
