#pragma once

#include "IOContext.h"

class LoadContextImp : public LoadContext {
public:
	struct LoadError {
		LoadError(Device const*d, const char *cstr) { device = d; msg.assign(cstr);};
		Device const*device;
		WMString msg;
	};
	struct LoadFilterEntry {
		Device *device;
		LoadFilter *filter;
	};

	LoadContextImp();
	~LoadContextImp();

	void setPresetState(bool loading_preset);
	void setPreserveID( bool preserve );

	virtual bool loadingPreset() override;
	virtual bool isPreservingID() override;

	/// Producing a load error will stop the world loading process, so use this only in scenarios
	/// where continuing would produce nonsense/crashes. This allows you to provide a message as to why the load is being stopped.
	virtual void errorOnLoad(Device const*dev, const char *msg) override;

	/// Warnings on load are very useful for notifying the user if your device has changed its method of operation in a
	/// non-transparent way. The LoadContext will collect all of these messages (usually displaying them to the user after load)
	virtual void warningOnLoad(Device const*dev, const char *msg) override;

	/// Call to register a patch with the WM load system. See documentation for LoadPatch; but essentially this allows you
	/// to do things like programmatically add additional devices to your input/output to replicate some old functionality,
	/// or re-wire inputs
	virtual void registerFilter(Device *dev, LoadFilter *filter) override;

	/// Call to register a port-swap patch with the WM load system. This can even swap wirings to a new device that has been dynamically created!
	virtual void registerPatch(PortInfoStruc const&requestedport, PortInfoStruc const&remappedport) override;

	// These methods are called by the owner of the load context to process any load filters that have been registered
	virtual void processFiltersDeviceLoaded(Device *dev);
	virtual void processLinkPatch( Device * &src, int &sp, int &st );
	virtual void processFiltersWorldLoaded();


	/// Any issues?
	bool hasErrors();
	bool hasWarnings();

	std::vector<LoadError, wmcore_allocator<LoadError> > &getErrors();
	std::vector<LoadError, wmcore_allocator<LoadError> > &getWarnings();
	std::vector<LoadFilterEntry, wmcore_allocator<LoadFilterEntry> > &getFilters();

private:
	struct LCImpl;
	std::unique_ptr<LCImpl> impl;
};

class SaveContextImpl : public SaveContext {
public:
	virtual bool saveForOldVersion(int &out_version) override;

};